@@ -127,7 +127,8 @@ gok_discard_focus_filter (GdkXEvent *xevent,
             }
         }
       if (!has_focus && n_mapped > 1)
-        XSetInputFocus (xev->xany.display, GDK_WINDOW_XID (GTK_WIDGET (window)->window), RevertToPointerRoot, CurrentTime);
+        XSetInputFocus (xev->xany.display, GDK_WINDOW_XID (GTK_WIDGET (window)->window), 
+			RevertToPointerRoot, CurrentTime);
 
       g_list_free (list);
       return GDK_FILTER_REMOVE;
@@ -192,11 +193,6 @@ on_window1_realize                     (GtkWidget       *widget,
 			 gok_discard_focus_filter,
 			 NULL);
 
-  if (!gok_main_check_sticky_keys (widget)) {
-	  gok_main_close ();
-	  gtk_exit (0);
-  }
-
 }
 
 
@@ -972,4 +968,3 @@ on_joystick_button_toggled             (GtkToggleButton *togglebutton,
 	gok_page_actions_set_is_corepointer (
 		!gtk_toggle_button_get_active (togglebutton));
 }
-
