@@ -117,7 +117,22 @@ void gok_chunker_initialize ()
 }
 
 /**
-* gok_chunker_rows
+* gok_chunker_chunk_none
+*
+* Makes sure all the keyboards have no chunks.
+*
+* returns: TRUE if everything went OK, FALSE if there was any problem.
+**/
+gboolean gok_chunker_chunk_none ()
+{	
+	gok_chunker_initialize();
+	return gok_chunker_chunk_all(CHUNKER_NOCHUNKS, 
+		CHUNKER_TOPTOBOTTOM_LEFTTORIGHT);
+}
+
+
+/**
+* gok_chunker_chunk_rows
 * @LeftRight: Direction the keys are ordered. 
 * @TopBottom: Direction the rows are ordered.
 *
@@ -155,7 +170,7 @@ gboolean gok_chunker_chunk_rows (gboolean TopBottom, gboolean LeftRight)
 }
 
 /**
-* gok_chunker_columns
+* gok_chunker_chunk_columns
 * @LeftRight: Direction the keys are ordered. 
 * @TopBottom: Direction the rows are ordered.
 *
@@ -297,7 +312,8 @@ gboolean gok_chunker_chunk (GokKeyboard* pKeyboard)
 
 	/* remove any current chunks on the keyboard */
 	gok_chunker_delete_chunks (pKeyboard->pChunkFirst, TRUE);
-
+	pKeyboard->pChunkFirst = NULL; /* very important */
+	
 	/* what type of chunks are required? */
 	switch (m_ChunkType)
 	{
@@ -1790,6 +1806,13 @@ void gok_chunker_delete_chunks (GokChunk* pChunk, gboolean bAlsoNext)
 		{
 			gok_chunker_delete_chunks (pChunkTemp->pChunkChild, TRUE);
 		}
+		
+		/* defensive */
+		if (pChunkTemp == m_pChunkHighlighted) {
+			gok_chunker_unhighlight_chunk (m_pChunkHighlighted);
+			m_pChunkHighlighted = NULL;
+		}
+		
 		g_free (pChunkTemp);
 		
 		/* keep deleting the following chunks? */
@@ -1807,6 +1830,8 @@ void gok_chunker_delete_chunks (GokChunk* pChunk, gboolean bAlsoNext)
 **/
 void gok_chunker_reset ()
 {
+	/* FIXME: remove all existing chunks ?? */
+	/* and call gok_chunker_initialize  ?? */
 }
 
 /**
@@ -2238,6 +2263,7 @@ void gok_chunker_unhighlight_all_keys ()
 	GokKeyboard* pKeyboard;
 	GokKey* pKey;
 	
+	gok_log_enter();
 	pKeyboard = gok_main_get_current_keyboard ();
 	if (pKeyboard != NULL)
 	{
@@ -2248,6 +2274,7 @@ void gok_chunker_unhighlight_all_keys ()
 			pKey = pKey->pKeyNext;
 		}
 	}
+	gok_log_leave();
 }
 
 /**
@@ -2258,8 +2285,11 @@ void gok_chunker_unhighlight_all_keys ()
 **/
 void gok_chunker_highlight_first_key ()
 {
-	g_assert (m_pChunkHighlighted != NULL);
-
+	if (m_pChunkHighlighted == NULL) {
+		gok_log_x( "aborting - no highlighted chunk to scan");
+		return;
+	}
+	
 	m_pChunkitemHighlighted = m_pChunkHighlighted->pChunkitem;
 	g_assert (m_pChunkitemHighlighted != NULL);
 	g_assert (m_pChunkitemHighlighted->pKey != NULL);
