@@ -119,6 +119,10 @@ void gok_data_set_dock_type (GokDockType dock_type);
 gint gok_data_get_repeat_rate (void);
 void gok_data_set_repeat_rate (gint rate);
 GConfClient* gok_data_get_gconf_client (void);
+const gchar* gok_data_get_aux_dictionaries (void);
+void gok_data_set_aux_dictionaries (const gchar *dictionaries);
+gboolean gok_data_get_use_aux_dictionaries (void);
+void gok_data_set_use_aux_dictionaries (gboolean use_aux);
 
 #ifdef __cplusplus
 }
