@@ -1163,3 +1163,13 @@ void gok_feedback_set_highlighted_key (GokKey* pKey)
 	m_pKeyHighlighted = pKey;
 }
 
+/* Temporary fix which make sure we aren't holding a pointer to a dead key */
+void
+gok_feedback_drop_refs (GokKey *pKey)
+{
+  if (m_pKeyflashKey == pKey) m_pKeyflashKey = NULL;
+  if (m_pKeySelected == pKey) m_pKeySelected = NULL;
+  if (m_pKeyHighlighted == pKey) m_pKeyHighlighted = NULL;
+}
+
+
