@@ -52,54 +52,50 @@ char m_label[XkbKeyNameLength + 1];
 
 gint m_Group = 0; /* FIXME: statics are bad */
 
+/* returned string must be freed */
 static char *
 gok_key_lookup_shorter_label (char *str)
 {
 	gint i;
+	gchar *xstr = NULL;
 
 	if (!strcmp (str, "Delete")) {
-		g_free (str);
-		str = "Del";
+		xstr = "Del";
 	}
 	else if (!strcmp (str, "Super")) {
-		g_free (str);
-		str = "Fn";
+		xstr = "Fn";
 	}
 	else if (!strcmp (str, "Insert")) {
-		g_free (str);
-		str = "Ins";
+		xstr = "Ins";
 	}
 	else if (!strcmp (str, "Control")) {
-		g_free (str);
-		str = "Ctrl";
+		xstr = "Ctrl";
 	}
 	else if (!strcmp (str, "Escape")) {
-		g_free (str);
-		str = "Esc";
+		xstr = "Esc";
 	}
 	else if (!strcmp (str, "SunAudioRaiseVolume")) {
-		g_free (str);
-		str = "Vol\n+";
+		xstr = "Vol\n+";
 	}
 	else if (!strcmp (str, "SunAudioLowerVolume")) {
-		g_free (str);
-		str = "Vol\n-";
+		xstr = "Vol\n-";
 	}
 	else if (!strcmp (str, "SunAudioMute")) {
-		g_free (str);
-		str = "Mute";
+		xstr = "Mute";
 	}
 	else if (!strcmp (str, "Pointer\nEnableKeys")) {
-		g_free (str);
-		str = "Mouse\nKeys";
+		xstr = "Mouse\nKeys";
 	}
 	else if (!strcmp (str, "ISO_Left\nTab")) {
-		g_free (str);
-		str = "Left\nTab";
+		xstr = "Left\nTab";
 	}
 	else if (!strcmp (str, "Sterling")) {
+		xstr = "£";
+	}
+
+	if (xstr != NULL) {
 		g_free (str);
-		str = "£";
+		str = g_strdup(xstr);
 	}
 	else {
 		gint len;
@@ -457,6 +453,7 @@ gok_key_from_xkb_key (GokKey *prevKey, GokKeyboard *pKeyboard, Display *display,
   gok_keylabel_new (pKey, label, group1_mask, group1_mask);
   label = gok_key_label_from_keycode (keycode, display, ShiftMask, 1);
   gok_keylabel_new (pKey, label, "shift+group1");*/
+  g_free(label);
   snprintf (keycode_name, 8, "%d", keycode);
   pKey->pOutput = gok_output_new (OUTPUT_KEYCODE, keycode_name, SPI_KEY_PRESSRELEASE);
   return pKey;
@@ -512,6 +509,7 @@ GokKey* gok_key_new (GokKey* pKeyPrevious, GokKey* pKeyNext, GokKeyboard* pKeybo
 	pgok_key_new->pOutputWrapperPost = NULL;
 	pgok_key_new->pButton = NULL;
 	pgok_key_new->ModifierName = NULL;
+	pgok_key_new->pButton = NULL;
 	pgok_key_new->Top = 0;
 	pgok_key_new->Bottom = 0;
 	pgok_key_new->Left = 0;
@@ -617,6 +615,7 @@ void gok_key_delete (GokKey* pKey, GokKeyboard* pKeyboard, gboolean bDeleteButto
 	if ((pKey->pButton != NULL) &&
 		(bDeleteButton == TRUE))
 	{
+/*		g_free(GOK_BUTTON(pKey->pButton)->indicator_type); */
 		gtk_widget_destroy (pKey->pButton);
 		pKey->pButton = NULL;
 	}
@@ -922,7 +921,8 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 		pStringAttributeValue = xmlGetProp (pNode, (const xmlChar *) "modifiertype");
 		if (pStringAttributeValue != NULL)
 		{
-			if (strcmp ((char *)pStringAttributeValue, "toggle") == 0)
+	
+if (strcmp ((char *)pStringAttributeValue, "toggle") == 0)
 			{
 				gok_modifier_set_type (pKey->ModifierName, MODIFIER_TYPE_TOGGLE);
 			}
@@ -950,7 +950,7 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 				}
 				g_strfreev (tokens);
 			}
-			gok_keylabel_new (pKey, (char *)xmlNodeGetContent (pNodeKeyChild),
+			gok_keylabel_new (pKey, (char *)g_strstrip (xmlNodeGetContent (pNodeKeyChild)),
 					  modifier, modmask);
 			pKey->has_text = TRUE;
 		}
@@ -1657,18 +1657,23 @@ void gok_key_set_button_name (GokKey* pKey)
 void
 gok_key_update_toggle_state (GokKey *pKey)
 {
+	gok_log_enter();
 	if (pKey && pKey->pButton && GTK_IS_TOGGLE_BUTTON (pKey->pButton)) {
-	        gboolean is_active = FALSE; 
-		if (pKey->ModifierName != NULL)
+		gboolean is_active = FALSE; 
+		if (pKey->ModifierName != NULL) {
 			is_active = 
-			(gok_modifier_get_state (pKey->ModifierName) != 0);
-		else 
+			(gok_modifier_get_state (pKey->ModifierName) != MODIFIER_STATE_OFF);
+		}
+		else {
 			is_active = pKey->ComponentState.active;
-		if (pKey->ComponentState.radio) 
-			GOK_BUTTON (pKey->pButton)->indicator_type = "radiobutton";
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pKey->pButton),
+		}
+		if (pKey->ComponentState.radio) {
+			GOK_BUTTON (pKey->pButton)->indicator_type = /*g_strdup( ?*/ "radiobutton" /*)*/;
+		}
+		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (pKey->pButton),
 					      is_active);
 	}
+	gok_log_leave();
 }
 
 /**
