@@ -123,12 +123,15 @@ KEYSTYLE_INSENSITIVE
 #define FONT_SIZE_GROUP_GLYPH -3
 
 #define GOK_MODMASK_CURRENT -1
+#define GOK_MIN_FONT_SIZE 6000 
+
 
 /* a key label */
 /* if you add data members to this structure, initialize them in gok_keylabel_new */
 typedef struct GokKeyLabel{
 	gchar* Text;
-	gchar* Modifier;
+        guint  modifiers;
+	guint  mod_mask;
 	struct GokKeyLabel* pLabelNext;	
 } GokKeyLabel;
 
@@ -219,7 +222,7 @@ typedef struct GokKey{
 } GokKey;
 
 gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode);
-gboolean gok_key_add_label (GokKey* pKey, gchar* pLabelText, gchar* pModifier);
+gboolean gok_key_add_label (GokKey* pKey, gchar* pLabelText, guint modifiers, guint mod_mask);
 void gok_key_set_output (GokKey* pKey, gint Type, gchar* pName, AccessibleKeySynthType Flag);
 void gok_key_add_output (GokKey* pKey, gint Type, gchar* pName, AccessibleKeySynthType Flag);
 void gok_key_change_label (GokKey* pKey, gchar* LabelText);
@@ -235,7 +238,7 @@ GtkWidget* gok_key_create_image_widget (GokKey* pKey);
 GokKeyImage* gok_key_get_image (GokKey* pKey);
 gchar* gok_key_get_image_filename (GokKey* pKey);
 gchar* gok_key_get_label (GokKey* pKey);
-GokKeyLabel* gok_keylabel_new (GokKey* pKey, gchar* pLabelText, gchar* pModifierText);
+GokKeyLabel* gok_keylabel_new (GokKey* pKey, gchar* pLabelText, guint modifiers, guint mod_mask);
 void gok_keylabel_delete (GokKeyLabel* pKeyLabel);
 void gok_key_update_toggle_state (GokKey *pKey);
 void gok_key_set_cells(GokKey* pKey, gint top, gint bottom, gint left, gint right);
