@@ -270,6 +270,39 @@ void gok_modifier_all_off ()
 {
 }
 
+/**
+ * gok_modifier_mask_for_name:
+ * Returns the modifier mask (single bit set) corresponding to @name, for
+ * a given @display.
+ **/
+guint
+gok_modifier_mask_for_name (Display *display, char *modifier_name)
+{
+	/* TODO: generalize this, instead of just hacking it in */
+	if (!modifier_name)
+		return SPI_KEYMASK_UNMODIFIED;
+	else if (!strcmp (modifier_name, "shift"))
+		return SPI_KEYMASK_SHIFT;
+	else if (!strcmp (modifier_name, "capslock") || !strcmp (modifier_name, "lock"))
+		return SPI_KEYMASK_SHIFTLOCK;
+	else if (!strcmp (modifier_name, "control"))
+		return SPI_KEYMASK_CONTROL;
+	else if (!strcmp (modifier_name, "alt"))
+		return SPI_KEYMASK_ALT;
+	else if (!strcmp (modifier_name, "numlock"))
+		return gok_key_get_numlock_mask (display);
+	else if (!strcmp (modifier_name, "mod1"))
+		return SPI_KEYMASK_MOD1;
+	else if (!strcmp (modifier_name, "mod2"))
+		return SPI_KEYMASK_MOD2;
+	else if (!strcmp (modifier_name, "mod3"))
+		return SPI_KEYMASK_MOD3;
+	else if (!strcmp (modifier_name, "mod4"))
+		return SPI_KEYMASK_MOD4;
+	else
+		return SPI_KEYMASK_UNMODIFIED;
+}
+
 /**
 * gok_modifier_get_state
 * @NameModifier: Name of the modifier you want the state for.
@@ -280,21 +313,28 @@ int gok_modifier_get_state (gchar* NameModifier)
 {
 	GokModifier* pModifier;
 
-	/* find the modifier in the list */
-	pModifier = m_pModifierFirst;
-	while (pModifier != NULL)
+	/* if this is one of the 'well-known' modifiers defined by Xlib, just check modmask */
+	if (gok_modifier_mask_for_name (gok_main_display (), NameModifier) & gok_spy_get_modmask ()) {
+		return TRUE;
+	}
+	else 
 	{
-		if ((pModifier->Name != NULL) &&
-			(strcmp (pModifier->Name, NameModifier) == 0))
+		/* find the modifier in the list */
+		pModifier = m_pModifierFirst;
+		while (pModifier != NULL)
 		{
-			/* return its state */
-			return pModifier->State;
+			if ((pModifier->Name != NULL) &&
+			    (strcmp (pModifier->Name, NameModifier) == 0))
+			{
+				/* return its state */
+				return pModifier->State;
+			}
+			pModifier = pModifier->pModifierNext;
 		}
-		pModifier = pModifier->pModifierNext;
+		
+		/* can't find the modifier in the list so return OFF */
+		gok_log_x ("Can't find modifier named %s!\n", NameModifier);
 	}
-
-	/* can't find the modifier in the list so return OFF */
-	gok_log_x ("Can't find modifier named %s!\n", NameModifier);
 	return MODIFIER_STATE_OFF;
 }
 
@@ -334,32 +374,8 @@ int gok_modifier_get_type (gchar* NameModifier)
 **/
 gboolean gok_modifier_get_normal()
 {
-	GokModifier* pModifier;
-
-	pModifier = m_pModifierFirst;
-	while (pModifier != NULL)
-	{
-		if (pModifier->State != MODIFIER_STATE_OFF)
-		{
-			return FALSE;
-		}
-		pModifier = pModifier->pModifierNext;
-	}
-	return TRUE;
-}
-
-/**
-* gok_modifier_press
-* @NameModifier: Name of the modifier key that has been pressed.
-*
-* This must be called when a modifier key has been pressed. Changes the
-* state of the modifier.
-**/
-void gok_modifier_press (GokKey* pKey, gchar* NameModifier)
-{
-	GokModifier* pModifier;
-	
-	gok_keyboard_output_key (pKey);
+	/* FIXME: too simple if and when we add GOK-specific modifiers that don't map to keys */
+	return (gok_spy_get_modmask () == 0);
 }
 
 /**
@@ -388,30 +404,11 @@ void gok_modifier_update_modifier_keys (GokKeyboard* pKeyboard)
 			state = gok_modifier_get_state (pKey->ModifierName);
 			if (state == MODIFIER_STATE_ON)
 			{
-#if STATUS_IN_LABEL
-				strcpy (buffer, gok_key_get_label (pKey));
-				if (gok_modifier_get_type (pKey->ModifierName) == MODIFIER_TYPE_NORMAL)
-				{
-					strcat (buffer, "+");
-				}
-				else
-				{
-					strcat (buffer, "!");
-				}
-				gok_key_set_button_label (pKey, buffer);
-#else
 				pKey->ComponentState.latched = 1;
-#endif
 			}
 			else if (state == MODIFIER_STATE_LOCKED)
 			{
-#if STATUS_IN_STRING
-				strcpy (buffer, gok_key_get_label (pKey));
-				strcat (buffer, "*");
-				gok_key_set_button_label (pKey, buffer);
-#else
 				pKey->ComponentState.locked = 1;
-#endif
 			}
 			gok_key_update_toggle_state (pKey);
 		}
@@ -419,37 +416,6 @@ void gok_modifier_update_modifier_keys (GokKeyboard* pKeyboard)
 	}
 }
 
-/**
-* gok_modifier_set_state:
-* @Name: Name of the modifier.
-* @State: New state of the modifier (see enum ModifierStates).
-*
-* Changes the state of the given modifier.
-**/
-void gok_modifier_set_state (gchar* Name, gint State)
-{
-	GokModifier* pModifier;
-
-	/* find the modifier in the list */
-	pModifier = m_pModifierFirst;
-	while (pModifier != NULL)
-	{
-		if (strcmp (pModifier->Name, Name) == 0)
-		{
-			if (pModifier->State != State)
-			{
-				pModifier->State = State;
-				
-				gok_keyboard_update_labels();
-				gok_modifier_update_modifier_keys (gok_main_get_current_keyboard());
-			}
-			return;
-		}
-		pModifier = pModifier->pModifierNext;
-	}
-	gok_log_x ("Can't find modifier: %s!\n", Name);
-}
-
 /**
  **/
 gchar*
