@@ -93,12 +93,10 @@ gok_output_update_predictions (GokOutput    *output,
 	}
 }
 
-gunichar gok_output_unichar_from_keysym (KeySym keysym)
+gunichar gok_output_unichar_from_keysym (KeySym keysym, guint mods)
 {
 	char *charstring;
-	if (keysym <= 0xff) 
-		return (gunichar) keysym;
-	else if ((keysym >= 0xFF08) && (keysym < 0xFF20)) { /* X11 control codes, etc. */
+	if ((keysym >= 0xFF08) && (keysym < 0xFF20)) { /* X11 control codes, etc. */
 		return (gunichar) keysym & 0x00FF; /* mask out the high bits */
 	}
 	else {
@@ -109,10 +107,10 @@ gunichar gok_output_unichar_from_keysym (KeySym keysym)
 		{
 			display = GDK_WINDOW_XDISPLAY (gok_main_get_main_window ()->window);
 			/* hmm, how do we know XkbTranslateKeySym will use UTF-8 ? */
-			/* TODO: pass current modmask instead of 0 (edge cases) */
 			nbytes = MIN (XkbTranslateKeySym (display,
-							  &keysym, 0, buf, 19, &extra_rtn), 19); 
+							  &keysym, mods, buf, 19, &extra_rtn), 19); 
 			if (nbytes) {
+			  fprintf (stderr, "unichar is [%s]\n", buf);
 				return g_utf8_get_char_validated (buf, nbytes);
 			}
 		}
@@ -275,7 +273,8 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 			/* update the word prediction keys with the new output */
 			if (bWordCompletion && gok_output_can_predict ()) {
 				gok_output_update_predictions (pOutput,
-							       (gpointer) gok_output_unichar_from_keysym (keysym));
+							       (gpointer) gok_output_unichar_from_keysym (keysym,
+													  gok_spy_get_modmask ()));
 			}
 		}
 		else if (pOutput->Type == OUTPUT_KEYCODE)
@@ -295,16 +294,19 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 				    gok_keyboard_get_xkb_desc (),
 				    keycode)) 
 			{
+			        /* FIXME this hack only works for Xlib core keymaps, not xkb-enhanced ones */
 				gulong mask = gok_spy_get_modmask ();
+				gint group = gok_key_get_effective_group ();
 			        int index = (mask & ShiftMask) ?
-					((mask & LockMask) ? 0 : 1) : 
-					((mask & LockMask) ? 1 : 0);
+					((group > 0) ? (group + 2) : (mask & LockMask) ? 0 : 1) : 
+					((group > 0) ? (group + 1) : (mask & LockMask) ? 1 : 0);
 
-				/* TODO: get the correct index from XKB */
+				/* TODO: get the correct index from XKB, that would be the right thing to do ;-) */
 			        keysym = XKeycodeToKeysym (gok_main_display (), 
 							   keycode, index);
 				gok_output_update_predictions (pOutput,
-							       (gpointer) gok_output_unichar_from_keysym (keysym));
+							       (gpointer) gok_output_unichar_from_keysym (keysym,
+													  gok_spy_get_modmask ()));
 			}
 		}
 		else if (pOutput->Type == OUTPUT_KEYSTRING)
