@@ -35,7 +35,8 @@ typedef enum {
 	OUTPUT_KEYSYM,
 	OUTPUT_KEYCODE,
 	OUTPUT_KEYSTRING,
-	OUTPUT_EXEC
+	OUTPUT_EXEC,
+	OUTPUT_INTERNAL
 } GokOutputType;
 
 /* an output */
@@ -50,6 +51,7 @@ void gok_output_delete_all (GokOutput* pOutput);
 GokOutput* gok_output_new (gint Type, gchar* Name, AccessibleKeySynthType Flag);
 GokOutput* gok_output_new_from_xml (xmlNode* pNode);
 void gok_output_send_to_system (GokOutput* pOutput, gboolean bNoWordCompletion);
+void gok_output_internal (gchar* action);
 
 #ifdef __cplusplus
 }
