@@ -62,6 +62,7 @@ gboolean gok_page_actions_initialize (GtkWidget* pWindowSettings)
 	GokAction* pAction;
 	GList* items = NULL;
 	GSList* pDevice = NULL;
+	GSList* slist;
 	gboolean bReturnCode;
 	
 	g_assert (pWindowSettings != NULL);
@@ -97,19 +98,36 @@ gboolean gok_page_actions_initialize (GtkWidget* pWindowSettings)
 	if (pDevice)
 	{
 		items = NULL;
-		while ((pDevice != NULL) && (pDevice->data != NULL))
+		slist = pDevice;
+		while ((slist != NULL) && (slist->data != NULL))
 		{
-	        	GokInput *input;
-			input = (GokInput *) pDevice->data;
-			items = g_list_append (items, input->name);
-			gok_input_free (input); 
+			GokInput *input;
+			input = (GokInput *) slist->data;
+			/* small hack so that the gconf stored extension device will
+			   be at the top of the list, and therefore not overridden 
+			   TODO: store this as part of an action */
+			if (strcmp(input->name, gok_input_get_extension_device_name()) == 0) {
+				items = g_list_prepend (items, input->name );
+			}
+			else {
+				items = g_list_append (items, input->name);
+			}
 			/* TODO: persist this list so we can query it later */
-			pDevice = pDevice->next;
+			slist = slist->next;
 		}
-		g_slist_free (pDevice);
 	
 		gtk_combo_set_popdown_strings (GTK_COMBO (pComboBox), items);
 		g_list_free (items);
+
+		slist = pDevice;
+		while ((slist != NULL) && (slist->data != NULL))
+		{
+			GokInput *input;
+			input = (GokInput *) slist->data;
+			gok_input_free (input); 
+			slist = slist->next;
+		}
+		g_slist_free (pDevice);
 	}
 
 	gok_page_actions_enable_radios_type (FALSE);
@@ -129,12 +147,13 @@ void gok_page_actions_refresh ()
 /**
 * gok_page_actions_apply
 * 
-* Updates the gok data with values from the controls.
+* Updates the gok data with values from the controls.  Unfinished.
 *
 * Returns: TRUE if any of the data members have changed, FALSE if not.
 **/
 gboolean gok_page_actions_apply ()
 {
+	/* TODO: finish implementation */	
 	return FALSE;
 }
 
@@ -704,25 +723,27 @@ void gok_page_actions_action_changed (GtkEditable* pEditControl)
 void gok_page_actions_input_device_changed (GtkEditable* pEditControl)
 {
 	gchar* pStrComboInputDeviceName;
+	GokInput *pInput;
 
 	/* get the name of the input device from the combo box */
 	pStrComboInputDeviceName = gtk_editable_get_chars (pEditControl, 0, -1);
-	if (strlen (pStrComboInputDeviceName) == 0)
-	{
+	if (strlen (pStrComboInputDeviceName) == 0) {
 		g_free (pStrComboInputDeviceName);
 	}
 	else
 	{
 		/* find the device in our list of input devices */
 	        
-		if (!gok_input_find_by_name (pStrComboInputDeviceName, TRUE))
-		{
+		pInput = gok_input_find_by_name (pStrComboInputDeviceName, TRUE);
+		if (!pInput) {
 			gok_log_x ("Input Device name (%s) from combo not found in device list!", pStrComboInputDeviceName);
 			gtk_editable_delete_text (pEditControl, 0, -1);
 		}
 		else {
+			gok_page_actions_set_changed ( TRUE );
 			gok_input_set_extension_device_by_name (
 				pStrComboInputDeviceName);
+			gok_input_free (pInput);
 		}
 		g_free (pStrComboInputDeviceName);
 	}
@@ -1076,5 +1097,3 @@ void gok_page_actions_set_changed (gboolean bTrueFalse)
 {
 	m_bChanged = bTrueFalse;
 }
-
-
