@@ -35,6 +35,7 @@
 #ifndef  SPI_KEYMASK_MOD4
   #define  SPI_KEYMASK_MOD4 Mod4Mask
 #endif
+#include "main.h"
 #include "gok-spy.h"
 #include "gok-log.h"
 #include "gok-modifier.h"
@@ -61,8 +62,6 @@
 /*
  * private prototypes
  */
-static gboolean gok_spy_is_menu_role (AccessibleRole role);
-
 static void gok_spy_focus_listener (const AccessibleEvent *event,
                                     void *user_data);
 
@@ -103,6 +102,12 @@ static void gok_spy_window_activate_listener (const AccessibleEvent *event,
 
 static void gok_spy_window_deactivate_listener (const AccessibleEvent *event,
                                                 void *user_data);
+												
+static void gok_spy_object_state_listener (const AccessibleEvent *event,
+                                                void *user_data);
+
+static gboolean gok_spy_is_visible (Accessible *accessible);
+												
 
 /*
  * gok-spy variables
@@ -112,6 +117,7 @@ static GQueue* eventQueue;
 static void* m_ptheAppChangeListener;
 static void* m_ptheWindowChangeListener;
 static void* m_ptheMouseButtonListener;
+static void* m_ptheStateChangeListener;
 
 /* pointer to the current window */
 static Accessible* m_ptheWindowAccessible;
@@ -125,6 +131,7 @@ static AccessibleDeviceListener* mouseListener;
 static AccessibleEventListener* modifierListener;
 static AccessibleEventListener* windowActivateListener;
 static AccessibleEventListener* windowDeactivateListener;
+static AccessibleEventListener* objectStateListener;
 static gboolean m_gokSpyOpen = FALSE;
 static gint explicitrefs;
 static gint explicitunrefs;
@@ -286,6 +293,7 @@ void gok_spy_open()
 		m_ptheMouseButtonListener =NULL;
 		m_ptheAppChangeListener = NULL;
 		m_ptheWindowChangeListener = NULL;
+		m_ptheStateChangeListener = NULL;
 		m_ptheWindowAccessible = NULL;
 		m_ptheAccessibleWithText = NULL;
 
@@ -301,6 +309,8 @@ void gok_spy_open()
 			gok_spy_window_activate_listener, NULL);
 		windowDeactivateListener = SPI_createAccessibleEventListener (
 			gok_spy_window_deactivate_listener, NULL);
+		objectStateListener = SPI_createAccessibleEventListener (
+			gok_spy_object_state_listener, NULL);
 
 		success = SPI_registerGlobalEventListener (focusListener, "focus:"); 
 		gok_log ("SPI_registerGlobalEventListener for focus events returned: %d" , success);
@@ -323,8 +333,8 @@ void gok_spy_open()
 		gok_log ("SPI_registerGlobalEventListener for window:deactivate returned: %d", success);
 
 		success = SPI_registerGlobalEventListener (
-			windowDeactivateListener, "window:destroy");
-		gok_log ("SPI_registerGlobalEventListener for window:destroy returned: %d", success);
+			objectStateListener, "object:state-changed");
+		gok_log ("SPI_registerGlobalEventListener for object:state-changed returned: %d", success);
 
 	}
 	gok_log_leave ();
@@ -384,6 +394,13 @@ void gok_spy_stop (void)
 		AccessibleEventListener_unref(windowDeactivateListener );
 		windowDeactivateListener = NULL;
 	}
+	if (objectStateListener != NULL)
+	{
+		result= SPI_deregisterGlobalEventListenerAll (objectStateListener);
+		gok_log("deregistering object state change listener returned: %d",result);
+		AccessibleEventListener_unref(objectStateListener );
+		objectStateListener = NULL;
+	}
 }
 	
 /** 
@@ -433,7 +450,7 @@ void gok_spy_close(void)
 * 
 * @callback: the listener to register
 **/ 
-void gok_spy_register_appchangelistener(AppChangeListener* callback)
+void gok_spy_register_appchangelistener(AccessibleChangeListener* callback)
 {
 	gok_log_x("this function deprecated, use gok_spy_register_windowchangelistener instead");
 	/* m_ptheAppChangeListener = callback; */
@@ -444,7 +461,7 @@ void gok_spy_register_appchangelistener(AppChangeListener* callback)
 * 
 * @callback: the listener to deregister
 **/ 
-void gok_spy_deregister_appchangelistener(AppChangeListener* callback)
+void gok_spy_deregister_appchangelistener(AccessibleChangeListener* callback)
 {
 	gok_log_x("this function deprecated, use gok_spy_deregister_windowchangelistener instead");
 	/* m_ptheAppChangeListener = NULL; */
@@ -455,7 +472,7 @@ void gok_spy_deregister_appchangelistener(AppChangeListener* callback)
 * 
 * @callback: the listener to register
 **/ 
-void gok_spy_register_windowchangelistener(AppChangeListener* callback)
+void gok_spy_register_windowchangelistener(AccessibleChangeListener* callback)
 {
 	m_ptheWindowChangeListener = callback;
 }
@@ -465,11 +482,37 @@ void gok_spy_register_windowchangelistener(AppChangeListener* callback)
 * 
 * @callback: the listener to deregister
 **/ 
-void gok_spy_deregister_windowchangelistener(AppChangeListener* callback)
+void gok_spy_deregister_windowchangelistener(AccessibleChangeListener* callback)
 {
 	m_ptheWindowChangeListener = NULL;
 }
 
+
+/** 
+* gok_spy_register_objectstatelistener
+* 
+* Will only notify if the state change is in the correct context. The context
+* is this: the state change must be happening within the currently focused
+* application, otherwise the callback is not notified.
+*
+* @callback: the listener to register
+**/ 
+void gok_spy_register_objectstatelistener(AccessibleChangeListener* callback)
+{
+	m_ptheStateChangeListener = callback;
+}
+
+/** 
+* gok_spy_deregister_objectstatelistener
+* 
+* @callback: the listener to deregister
+**/ 
+void gok_spy_deregister_objectstatelistener(AccessibleChangeListener* callback)
+{
+	m_ptheStateChangeListener = NULL;
+}
+
+
 /** 
 * gok_spy_register_mousebuttonlistener
 * 
@@ -639,9 +682,16 @@ gboolean gok_spy_check_queues(void)
 		switch (en->type)
 		{
 		case GOKSPY_FOCUS_EVENT:
+			/* TODO: remove obsolete focus events from queue since process
+			   accessible is expensive. */
 			gok_log("GOKSPY_FOCUS_EVENT");
 			gok_spy_process_accessible(en->paccessible);
 			break;
+		case GOKSPY_STATE_EVENT:
+			gok_log("GOKSPY_STATE_EVENT");
+			/* gok_spy_accessible_ref (en->paccessible); */
+			((AccessibleChangeListenerCB)(m_ptheStateChangeListener))(en->paccessible);
+			break;
 		case GOKSPY_WINDOW_ACTIVATE_EVENT:
 			gok_log("GOKSPY_WINDOW_ACTIVATE_EVENT");
 			/* FIXME !! the ref needs to happen in the original event loop */
@@ -649,17 +699,18 @@ gboolean gok_spy_check_queues(void)
 				gok_spy_accessible_unref (m_ptheWindowAccessible);
 				m_ptheWindowAccessible = en->paccessible;
 				gok_spy_accessible_ref (m_ptheWindowAccessible);
-				((AppChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
+				((AccessibleChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
 			}
 			break;
 		case GOKSPY_DEFUNCT_EVENT:
 			gok_log("GOKSPY_DEFUNCT_EVENT");
 			/* note: currently only defunct events for the current application 
 				are pushed on the queue */
-			gok_spy_accessible_unref (m_ptheWindowAccessible); /* for our 
-				reference book-keeping */
+			gok_spy_accessible_unref (m_ptheWindowAccessible);
+			gok_spy_accessible_unref (m_ptheAccessibleWithText);
 			m_ptheWindowAccessible = NULL;
-			((AppChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
+			m_ptheAccessibleWithText = NULL;
+			((AccessibleChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
 			break;
 		case GOKSPY_WINDOW_DEACTIVATE_EVENT:
 			gok_log("GOKSPY_WINDOW_DEACTIVATE_EVENT");
@@ -674,8 +725,10 @@ gboolean gok_spy_check_queues(void)
 			 */
 			if (en->paccessible == m_ptheWindowAccessible) {
 				gok_spy_accessible_unref (m_ptheWindowAccessible);
+				gok_spy_accessible_unref (m_ptheAccessibleWithText);
 				m_ptheWindowAccessible = NULL;
-				((AppChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
+				m_ptheAccessibleWithText = NULL;
+				((AccessibleChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible);
 			}
 			break;
 		default:
@@ -700,13 +753,18 @@ gboolean gok_spy_check_queues(void)
 */ 
 gboolean gok_spy_check_window(AccessibleRole role)
 {
-	/* TODO - improve efficiency here? */
+	/* TODO - improve efficiency here? Also, roles get added and we need
+	   to maintain this...  maybe we need to go about this differently */
 	if ((role == SPI_ROLE_WINDOW) ||
 	    (role == SPI_ROLE_DIALOG) ||
 	    (role == SPI_ROLE_FILE_CHOOSER) ||
 	    (role == SPI_ROLE_FRAME) ||	
 	    (role == SPI_ROLE_DESKTOP_FRAME) ||
-		(role == SPI_ROLE_FONT_CHOOSER))
+		(role == SPI_ROLE_FONT_CHOOSER)	||
+		(role == SPI_ROLE_COLOR_CHOOSER) ||
+		(role == SPI_ROLE_APPLICATION) ||
+		(role == SPI_ROLE_ALERT)
+		)
 	{
 		return TRUE;
 	}
@@ -726,11 +784,12 @@ gboolean gok_spy_check_window(AccessibleRole role)
 gboolean gok_spy_worth_searching (Accessible* accessible, AccessibleRole role)
 {
 	gboolean worthiness;
+	gboolean bmenu;
 	AccessibleStateSet* ass = NULL;
 
 	gok_log_enter();
-	
-	if (gok_spy_is_menu_role (role))
+	bmenu = gok_spy_is_menu_role (role);
+	if (bmenu)
 	{		
 		switch (Accessible_getRole(accessible))
 		{
@@ -763,10 +822,13 @@ gboolean gok_spy_worth_searching (Accessible* accessible, AccessibleRole role)
 	if (ass != NULL)
 	{
 		AccessibleRole role = Accessible_getRole (accessible);
-		/* assumption: if this thing manages its own children, then it 
+		/* state heuristic: 
+		 * - if this thing manages its own children, then it 
 		 * is probably not worth traversing.
+		 * - if it is a menu-like thing then it doesn't need to be "showing"
 		 */
-		if ( AccessibleStateSet_contains( ass, SPI_STATE_MANAGES_DESCENDANTS ))
+		if (AccessibleStateSet_contains( ass, SPI_STATE_MANAGES_DESCENDANTS ) ||
+			(!bmenu && !AccessibleStateSet_contains( ass, SPI_STATE_SHOWING )) )
 		{
 			worthiness = FALSE;
 		}
@@ -788,8 +850,8 @@ gboolean gok_spy_worth_searching (Accessible* accessible, AccessibleRole role)
  * (pass in zero).
  * @found: out variable to hold the success boolean.
  * 
- * If anything under this accessible* has the specified role, then
- * returns true.
+ * If anything under this accessible* has the specified role (constrained by
+ * depth), then returns true.
  *
  * Returns: true if a child exists, false otherwise.
  */
@@ -802,6 +864,7 @@ gboolean gok_spy_has_child_aux (Accessible* accessible,
     AccessibleStateSet *states;
     int child_count;
     int i; 
+	gboolean nameless_menu = FALSE;
 
     if (accessible == NULL) return *found; 
 	if (cdepth++ == depth) return *found;
@@ -830,24 +893,28 @@ gboolean gok_spy_has_child_aux (Accessible* accessible,
 		    {
 			    if (Accessible_getRole (pAccessibleChild) == role) {
 				    gok_log ("gok_spy_has_child_aux: found child at depth %d", cdepth);
-				    gok_spy_accessible_unref(pAccessibleChild);
-				    *found = TRUE;
-				    return *found;
+					*found = TRUE;
+					gok_spy_accessible_unref(pAccessibleChild);
+					return *found;
 			    }
 		    }
 		    else if (type == GOK_SPY_SEARCH_MENU) {
 			    AccessibleRole a_role = Accessible_getRole (pAccessibleChild);
 			    if ((a_role == role) || 
-				gok_spy_is_menu_role (a_role)) {
+				 (a_role == SPI_ROLE_MENU)) {
 				    gchar *name = Accessible_getName (pAccessibleChild);
 				    /* menu items must have either text or an image! */
 				    if (name && (strlen (name) > 0)) { /* todo: what about icon-menus? */
-					    gok_log ("gok_spy_has_child_aux: found child at depth %d", cdepth);
+					    gok_log ("gok_spy_has_child_aux: found [%s] at depth %d", name, cdepth);
 					    gok_spy_accessible_unref(pAccessibleChild);
 					    SPI_freeString (name);
 					    *found = TRUE;					    
 					    return *found;
 				    }
+					else {
+						nameless_menu = TRUE;
+					}
+						
 			    }
 		    }
 		    else if (type == GOK_SPY_SEARCH_EDITABLE_TEXT) {
@@ -857,7 +924,11 @@ gboolean gok_spy_has_child_aux (Accessible* accessible,
 				    return *found;
 			    }
 		    }
-		    gok_spy_has_child_aux (pAccessibleChild, type, role, children, depth, cdepth, found);
+			/* ignore children of a nameless menu, and abort if we have already
+			   found one target */
+			if (!*found && !nameless_menu )
+		    	gok_spy_has_child_aux (pAccessibleChild, type, role, 
+					children, depth, cdepth, found);
 		    gok_spy_accessible_unref(pAccessibleChild);
 		} 
 	}
@@ -879,7 +950,7 @@ gok_spy_is_visible (Accessible *accessible)
 	return retval;
 }
 
-static gboolean
+gboolean
 gok_spy_is_menu_role (AccessibleRole role)
 {
 	return ((role  ==  SPI_ROLE_MENU_ITEM) ||
@@ -1123,18 +1194,21 @@ AccessibleNode* gok_spy_append_node (AccessibleNode* pNode,
 		AccessibleHypertext_unref (hypertext);
 	}
 
-	/* if the name is empty then try LABELLED_BY, else ignore this child */ 
+	/* if the name is empty then try LABELLED_BY, else assign "???" */ 
 	else {
 		/* look for a label */
 		if (pName == NULL)
 		{
 			gok_log("no name, so looking at relations...");
 			relations = Accessible_getRelationSet(pAccessible);
+			/*
 			if (relations == NULL)	{
 				gok_log("no relationset for this object");
 				gok_log_leave();
 				return pNewNode;
 			}
+			*/
+			if (relations != NULL)
 			while (relations[i]) {
 				type = AccessibleRelation_getRelationType(relations[i]);
 				if (type == SPI_RELATION_LABELED_BY) {
@@ -1163,12 +1237,43 @@ AccessibleNode* gok_spy_append_node (AccessibleNode* pNode,
 			}
 		}
 		
-		
+		if (pName == NULL) {
+			gok_log("still no name, so looking at description...");
+			/* one last try, we pull chars from description */
+			s = Accessible_getDescription (pAccessible);
+			if (s != NULL) {
+				gint len = strlen (s);
+				if ( len != 0 )  {
+					if (len > 18) 
+					{
+						gok_log("shortening description");
+						gchar *tmp = g_strndup (s, 15);
+						pName = g_strconcat (tmp, "...", NULL);
+						g_free (tmp);
+					}
+					else 
+					{
+						gok_log("using description");
+						pName = g_strndup (s, len);
+					}
+				}
+				SPI_freeString (s);
+			}
+		}
 		if (pName != NULL) {
+			gok_log("we have a name: [%s]",pName);
 			/* create a new node */
 			gok_spy_add_node (pNode, pAccessible, -1, pName);
 		}
+		/* if we decide to allow access to nameless menus ...
+		else if ( Accessible_getRole (pAccessible) == SPI_ROLE_MENU ) {
+			pName = g_strdup (Accessible_getRoleName(pAccessible));
+			gok_log ("Can't think of a good name so using role:[%s]",pName);
+			gok_spy_add_node (pNode, pAccessible, -1, pName);
+		}
+		*/
 		else if (Accessible_isText (pAccessible)) {
+			gok_log("still no name, but there is text...");
 			/* 
 			* sadly, many text objects are still nameless - yet we want 
 			* to allow the user to get to them.
@@ -1220,11 +1325,14 @@ AccessibleNode* gok_spy_append_node (AccessibleNode* pNode,
 				}
 			}
 			if (pName != NULL) {
+				gok_log ("we have a name: [%s]",pName);
 				gok_spy_add_node (pNode, pAccessible, -1, pName);
 			}
 		}
+			
 	}
 	
+	
 	while (pNode->pnext) 
 		pNode = pNode->pnext;
 
@@ -1243,7 +1351,6 @@ void gok_spy_focus_listener (const AccessibleEvent *event, void *user_data)
 	EventNode* en; 
 	gok_log_enter();
 	gok_spy_accessible_ref(event->source);
-	
 	en = (EventNode*)g_malloc(sizeof(EventNode));
 	en->paccessible = event->source;
 	en->type = GOKSPY_FOCUS_EVENT;
@@ -1309,6 +1416,24 @@ gok_spy_window_deactivate_listener(const AccessibleEvent *event,
 	gok_log_leave ();
 }
 
+
+static void
+gok_spy_object_state_listener(const AccessibleEvent *event,
+                                 void *user_data)
+{
+	EventNode *en;
+	gok_log_enter ();
+
+	gok_spy_accessible_ref (event->source);
+	en = (EventNode*)g_malloc (sizeof (EventNode));
+	en->paccessible = event->source;
+	en->type = GOKSPY_STATE_EVENT;
+	g_queue_push_tail (eventQueue, en);
+	gok_spy_add_idle_handler ();
+
+	gok_log_leave ();
+}
+
 /** 
 * gok_spy_process_accessible 
 * 
@@ -1390,7 +1515,7 @@ void gok_spy_process_accessible (Accessible* accessible)
 		gok_log("different.");
 		gok_spy_accessible_unref (m_ptheWindowAccessible);
 		m_ptheWindowAccessible = pnewWindowAccessible;
-		((AppChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible); 
+		((AccessibleChangeListenerCB)(m_ptheWindowChangeListener))(m_ptheWindowAccessible); 
 	}
 	else
 	{
@@ -1495,22 +1620,8 @@ void gok_spy_modifier_listener(const AccessibleEvent *event, void *user_data)
 
 	gok_log_enter();
 	gok_log ("event->detail2 = %d", event->detail2);
-
-	
-	for (m = gok_modifier_keymasks; m->name != NULL; m++)
-	{
-		if (event->detail2 & m->keymask)
-		{
-			gok_log ("%s on", m->name);
-			gok_modifier_set_state (m->name, MODIFIER_STATE_ON);
-		}
-		else
-		{
-			gok_log ("%s off", m->name);
-			gok_modifier_set_state (m->name, MODIFIER_STATE_OFF);
-		}
-	}
-
+	gok_keyboard_update_labels ();
+	gok_modifier_update_modifier_keys (gok_main_get_current_keyboard ());
 	gok_log_leave();
 }
 
@@ -1524,6 +1635,74 @@ void gok_spy_add_idle_handler (void)
 	g_idle_add (gok_spy_idle_handler, NULL);
 }
 
+/* 
+* gok_spy_get_active_frame
+* 
+* User must call gok_spy_free when finished with this list.
+* 
+* Returns: pointer to the active Accessible* or NULL
+*/
+Accessible* 
+gok_spy_get_active_frame( )
+{
+	gint i;
+	gint j;
+	gint k;
+	gint n_desktops;
+	gint n_apps;
+	gint n_frames;
+	Accessible* desktop;
+	Accessible* child;
+	AccessibleStateSet* ass = NULL;
+
+	gok_log_enter();
+
+	n_desktops = SPI_getDesktopCount();
+	for (i = 0; i < n_desktops; i++)
+	{
+		desktop = SPI_getDesktop(i);
+		n_apps= Accessible_getChildCount(desktop);
+		for (j = 0; j < n_apps; j++)
+		{
+			child = Accessible_getChildAtIndex(desktop, j);
+			
+			if (!child)
+				continue;
+			
+			/* Applications are not active
+			   so we must look for a child frame */
+			n_frames = Accessible_getChildCount (child);
+			for (k = 0; k < n_frames; k++) {
+				Accessible *frame;		
+				frame = Accessible_getChildAtIndex (child, k);
+
+				if (!frame)
+					continue;
+
+				ass = Accessible_getStateSet (frame); /* implicit ref? */
+				
+				if (!ass) {
+					Accessible_unref (frame);
+					continue;
+				}
+
+				if (AccessibleStateSet_contains (ass, SPI_STATE_ACTIVE)) {
+					gok_log ("Found active frame");
+					Accessible_unref (desktop);
+					AccessibleStateSet_unref (ass);
+					Accessible_unref (child);
+					return frame;
+				}
+				AccessibleStateSet_unref (ass);
+				Accessible_unref (frame);
+			}
+			Accessible_unref (child);
+		}
+		Accessible_unref (desktop);
+	}
+	gok_log_leave();
+	return NULL;
+}
 
 /*
 gboolean gok_spy_is_desktop(Accessible* pAccessible)
@@ -1598,54 +1777,3 @@ gboolean gok_spy_accessible_is_desktopChild(Accessible* accessible)
 	gok_log_leave();
 }
 */
-/* 
-* gok_spy_get_applications 
-* 
-* User must call gok_spy_free when finished with this list.
-* 
-* Returns: pointer to the list or NULL
-*/
-/*
-AccessibleNode* gok_spy_get_applications( )
-{
-	gint i;
-	gint j;
-	gint ndesktops;
-	gint nchildren;
-	Accessible* desktop;
-	Accessible* child;
-	AccessibleNode* plist;
-
-	gok_log_enter();
-
-	plist = NULL;
-	ndesktops = SPI_getDesktopCount();
-	for (i = 0; i < ndesktops; i++)
-	{
-		desktop = SPI_getDesktop(i);
-		gok_spy_accessible_implicit_ref(desktop);
-		if (desktop == NULL)
-		{
-			gok_log("desktop disappeared!?");
-			break;
-		}
-		nchildren = Accessible_getChildCount(desktop);
-		for (j = 0; j < nchildren; j++)
-		{
-			child = Accessible_getChildAtIndex(desktop, j);
-			gok_spy_accessible_implicit_ref(child);
-			if (child != NULL)
-			{
-				plist = gok_spy_append_node(plist, child);
-				if (plist == NULL)
-				{
-					gok_spy_accessible_unref(child);
-				}
-			}
-		}
-		gok_spy_accessible_unref(desktop);
-	}
-	gok_log_leave();
-	return plist;
-}
-*/
