@@ -41,6 +41,7 @@
 #include "gok-keyboard.h"
 
 #include "word-complete.h" /* TODO: remove this when we make this class abstract */
+#include "gok-utf8-word-complete.h"
 
 static void gok_wordcomplete_finalize (GObject *obj);
 static void gok_wordcomplete_real_reset (GokWordComplete *complete);
@@ -51,12 +52,15 @@ static GokWordCompletionCharacterCategory gok_wordcomplete_real_unichar_get_cate
 	                                         GokWordComplete *complete, gunichar letter);
 static gchar **gok_wordcomplete_real_predict (GokWordComplete *complete, gint num_predictions);
 static gchar *gok_wordcomplete_real_get_word_part (GokWordComplete *complete);
-
+static gchar *  gok_wordcomplete_real_get_aux_dict (GokWordComplete *complete);
+static gboolean gok_wordcomplete_real_set_aux_dict (GokWordComplete *complete, const gchar *dict_path);
 
 struct _GokWordCompletePrivate 
 {
         /* the word, or part word, that is getting completed */
 	gchar     *word_part;
+	gunichar     compose;
+	const gchar *aux_dictionaries;
 };
 
 static GokWordComplete *default_wordcomplete_engine = NULL;
@@ -69,6 +73,8 @@ gok_wordcomplete_instance_init (GokWordComplete *complete)
 {
 	complete->priv = g_new0 (GokWordCompletePrivate, 1);
 	complete->priv->word_part = NULL;
+	complete->priv->compose = 0;
+	complete->priv->aux_dictionaries = NULL;
 }
 
 static void
@@ -87,6 +93,8 @@ gok_wordcomplete_class_init (GokWordCompleteClass *klass)
 	klass->unichar_push = gok_wordcomplete_real_unichar_push;
 	klass->unichar_pop = gok_wordcomplete_real_unichar_pop;
 	klass->get_word_part = gok_wordcomplete_real_get_word_part;
+	klass->set_aux_dictionary = gok_wordcomplete_real_set_aux_dict;
+	klass->get_aux_dictionary = gok_wordcomplete_real_get_aux_dict;
 }
 
 static void
@@ -303,6 +311,36 @@ gok_wordcomplete_increment_word_frequency (GokWordComplete *complete, const gcha
 		return FALSE;
 }
 
+/**
+* gok_wordcomplete_get_aux_dictionaries:
+* 
+* returns: a fully-delimited file path to an auxiliary word prediction dictionary
+**/
+gchar*   
+gok_wordcomplete_get_aux_dictionaries (GokWordComplete *complete)
+{
+	GokWordCompleteClass *klass = GOK_WORDCOMPLETE_GET_CLASS (complete);
+	if (*klass->get_aux_dictionary)
+		return (* klass->get_aux_dictionary)(complete);
+	else
+		return NULL;
+}
+
+/**
+* gok_wordcomplete_set_aux_dictionaries:
+* 
+* returns: TRUE if an aux dictionary can be used by the word-completion engine, FALSE otherwise.
+**/
+gboolean 
+gok_wordcomplete_set_aux_dictionaries (GokWordComplete *complete, const gchar *dictionary_path)
+{
+	GokWordCompleteClass *klass = GOK_WORDCOMPLETE_GET_CLASS (complete);
+	if (*klass->set_aux_dictionary)
+		return (* klass->set_aux_dictionary)(complete, dictionary_path);
+	else
+		return FALSE;
+}
+
 /**
  * gok_wordcomplete_get_default:
  *
@@ -312,12 +350,26 @@ GokWordComplete*
 gok_wordcomplete_get_default (void)
 {
 	if (default_wordcomplete_engine == NULL)
-		default_wordcomplete_engine = g_object_new (GOK_TYPE_TRIEWORDCOMPLETE, NULL);  
+		default_wordcomplete_engine = g_object_new (GOK_TYPE_UTF8WORDCOMPLETE, NULL);  
         /* TODO: establish default subtype and create type - making the base class abstract */
 	return default_wordcomplete_engine;
 }
 
 /* private methods */
+
+static gboolean
+gok_wordcomplete_unichar_is_compose (GokWordComplete *complete, gunichar letter)
+{
+	GUnicodeType utype = g_unichar_type (letter);
+	return (utype == G_UNICODE_MODIFIER_LETTER || utype == G_UNICODE_MODIFIER_SYMBOL);
+}
+
+static gboolean
+gok_wordcomplete_unichar_is_combine (GokWordComplete *complete, gunichar letter)
+{
+	return (g_unichar_type (letter) == G_UNICODE_COMBINING_MARK);
+}
+
 static gboolean 
 gok_wordcomplete_unichar_is_backspace (GokWordComplete *complete, gunichar letter) 
 {
@@ -328,14 +380,146 @@ static gboolean
 gok_wordcomplete_unichar_is_delimiter (GokWordComplete *complete, gunichar letter)
 {		
 /* 
- * Is this whitespace, empty, punctuation, or nonprintable? Also catches NULL/0 
+ * Is this whitespace, empty, punctuation, or a control character? Also catches NULL/0 
  * we make exception for U+00a0 (nonbreak space) so 'words' containing 
- * space can be handled. 
+ * space can be handled. We also make some decisions based on the unicode line-breaking type;
+ * in general if a line break is allowed (except HYPHEN) after or between the adjacent 
+ * characters, we take that behavior to constitute a 'delimiter' for the purposes of
+ * bounding our word completion input.
  */
-	return (!g_unichar_validate (letter) ||  /* if invalid, */
-		((g_unichar_ispunct (letter) || !g_unichar_isgraph (letter)) /* or punct/non-graphical, */
-		 && (letter != 0x00a0)) || /* but not nbspace */
-		(letter == 0x2f)); /* or if destructive backspace */
+	GUnicodeBreakType break_type;
+
+	if ((!g_unichar_validate (letter) ||  /* if invalid, */
+	     g_unichar_iscntrl (letter)) /* or a control char, */
+	    && (letter != 0x00a0))  /* but not nbspace */
+		return TRUE;
+
+	break_type = g_unichar_break_type (letter);
+	switch (break_type)
+	{
+	case G_UNICODE_BREAK_CARRIAGE_RETURN:
+	case G_UNICODE_BREAK_LINE_FEED:
+	case G_UNICODE_BREAK_CONTINGENT:
+	case G_UNICODE_BREAK_SPACE:
+	case G_UNICODE_BREAK_INFIX_SEPARATOR:
+	case G_UNICODE_BREAK_SYMBOL:
+	case G_UNICODE_BREAK_OPEN_PUNCTUATION:
+	case G_UNICODE_BREAK_CLOSE_PUNCTUATION:
+	case G_UNICODE_BREAK_QUOTATION:
+	case G_UNICODE_BREAK_EXCLAMATION:
+	case G_UNICODE_BREAK_MANDATORY:
+	case G_UNICODE_BREAK_AFTER:
+	case G_UNICODE_BREAK_BEFORE_AND_AFTER:
+		return TRUE;
+	case G_UNICODE_BREAK_BEFORE:
+	case G_UNICODE_BREAK_NON_STARTER:
+	case G_UNICODE_BREAK_IDEOGRAPHIC:
+	case G_UNICODE_BREAK_NUMERIC:
+	case G_UNICODE_BREAK_ALPHABETIC:
+	case G_UNICODE_BREAK_PREFIX:
+	case G_UNICODE_BREAK_POSTFIX:
+	case G_UNICODE_BREAK_COMPLEX_CONTEXT:
+	case G_UNICODE_BREAK_HYPHEN:
+	case G_UNICODE_BREAK_COMBINING_MARK:
+	case G_UNICODE_BREAK_SURROGATE:
+	case G_UNICODE_BREAK_ZERO_WIDTH_SPACE:
+	case G_UNICODE_BREAK_INSEPARABLE:
+	case G_UNICODE_BREAK_NON_BREAKING_GLUE:
+	case G_UNICODE_BREAK_UNKNOWN:
+	case G_UNICODE_BREAK_AMBIGUOUS:
+	default:
+		return FALSE;
+	}
+}
+
+static gunichar
+gok_wordcomplete_compose (GokWordComplete *complete, gunichar combining_letter, gunichar base_letter)
+{
+	gchar *tmp, *norm, utf8char[7], utf8combining[7];
+	gunichar composed_letter = base_letter;
+
+	fprintf (stderr, "combining %c [%x], %c\n", combining_letter, combining_letter, base_letter);
+
+	/* we'll combine regardless of the order, assuming that the caller has kept track of that.
+	 * certain unicode characters are officially non-combining, 
+	 * but result from X compose keysym sequences. 
+	 * We will convert these into their equivalent unicode combining forms, and then use
+	 * utf8 normalization routines to convert them into their composed forms.
+	 */
+	if (combining_letter >= 0xa8 && combining_letter <= 0xb8 || combining_letter == 0x05e) 
+	{
+		switch (combining_letter)
+		{
+		case 0x5e: /* caret/circumflex */
+			combining_letter = 0x0302; /* combining circumflex */
+			break;
+		case 0xa8: /* dieresis */
+			combining_letter = 0x0308; /* combining dieresis */
+			break;
+		case 0xaf: /* macron */
+			combining_letter = 0x0304; /* combining macron */
+			break;
+		case 0xb0: /* degree sign */
+			combining_letter = 0x030a; /* combining ring above */
+			break;
+		case 0xb4: /* acute accent */
+			combining_letter = 0x0301; /* combining acute accent */
+			break;
+		case 0xb8: /* cedilla */
+			combining_letter = 0x0327; /* etc. */
+			break;
+		default:
+			break;
+		}
+	}
+	utf8char[g_unichar_to_utf8 (base_letter, utf8char)] = '\0';
+	utf8combining[g_unichar_to_utf8 (combining_letter, utf8combining)] = '\0';
+	tmp = g_strconcat (utf8char, utf8combining, NULL);
+	if (tmp) 
+	{
+		norm = g_utf8_normalize (tmp, -1, G_NORMALIZE_DEFAULT_COMPOSE); /* maximally composed forms */
+		g_free (tmp);
+		if (norm) 
+		{
+			composed_letter = g_utf8_get_char (norm);
+			g_free (norm);
+		}
+	}
+	return composed_letter;
+}
+
+static void
+gok_wordcomplete_compose_push (GokWordComplete *complete, gunichar combining_letter)
+{
+	complete->priv->compose = combining_letter;
+}
+
+static gunichar
+gok_wordcomplete_last_combine (GokWordComplete *complete, gunichar combining_letter)
+{
+	gunichar last_letter = 0, letter = 0;
+	gchar *word = gok_wordcomplete_get_word_part (complete);
+	if (word)
+	{
+		gchar utf8char[7];
+		gchar *tmp, *tmp2;
+		utf8char[g_unichar_to_utf8 (combining_letter, utf8char)] = '\0'; /* null terminate utf8 string */
+		tmp = g_strconcat (word, utf8char);
+		if (tmp)
+		{
+			gint len;
+			tmp2 = g_utf8_normalize (tmp, -1, G_NORMALIZE_DEFAULT_COMPOSE);
+			g_free (tmp);
+			if (tmp2) 
+			{
+				len = g_utf8_strlen (tmp2, -1);
+				if (len)
+					letter = g_utf8_get_char (g_utf8_offset_to_pointer (tmp2, len - 1));
+				g_free (tmp2);
+			}
+		}
+	}
+	return letter;
 }
 
 /* implementations of GokWordCompleteClass methods */
@@ -355,6 +539,11 @@ gok_wordcomplete_real_unichar_push (GokWordComplete *complete, gunichar letter)
 	if (complete && complete->priv) 
 	{
 		word_part = gok_wordcomplete_get_word_part (complete);
+		if (complete->priv->compose)
+		{
+			letter = gok_wordcomplete_compose (complete, complete->priv->compose, letter);
+			complete->priv->compose = 0;
+		}
 		utf8char [g_unichar_to_utf8 (letter, utf8char)] = '\0';
 		if (word_part) 
 		{
@@ -395,6 +584,10 @@ gok_wordcomplete_real_unichar_get_category (GokWordComplete *complete, gunichar
 {
 	if (gok_wordcomplete_unichar_is_backspace (complete, letter))
 		return GOK_WORDCOMPLETE_CHAR_TYPE_BACKSPACE;
+	else if (gok_wordcomplete_unichar_is_compose (complete, letter))
+		return GOK_WORDCOMPLETE_CHAR_TYPE_COMPOSE;
+	else if (gok_wordcomplete_unichar_is_combine (complete, letter))
+		return GOK_WORDCOMPLETE_CHAR_TYPE_COMBINE;
 	else if (gok_wordcomplete_unichar_is_delimiter (complete, letter))
 		return GOK_WORDCOMPLETE_CHAR_TYPE_DELIMITER;
 	else
@@ -404,7 +597,9 @@ gok_wordcomplete_real_unichar_get_category (GokWordComplete *complete, gunichar
 static gchar *
 gok_wordcomplete_real_process_unichar (GokWordComplete *complete, gunichar letter)
 {
-	gchar *prefix = NULL;
+	gchar *prefix = NULL, *tmpstring = NULL;
+	gchar utf8char[7];
+
 	switch (gok_wordcomplete_unichar_get_category (complete, letter))
 	{
 	case GOK_WORDCOMPLETE_CHAR_TYPE_BACKSPACE:
@@ -413,6 +608,13 @@ gok_wordcomplete_real_process_unichar (GokWordComplete *complete, gunichar lette
 	case GOK_WORDCOMPLETE_CHAR_TYPE_DELIMITER:
 		gok_wordcomplete_reset (complete);
 		break;
+	case GOK_WORDCOMPLETE_CHAR_TYPE_COMPOSE:
+		gok_wordcomplete_compose_push (complete, letter);
+		break;
+	case GOK_WORDCOMPLETE_CHAR_TYPE_COMBINE:
+		letter = gok_wordcomplete_last_combine (complete, letter);
+		gok_wordcomplete_unichar_pop (complete);
+		/* fall through */
 	case GOK_WORDCOMPLETE_CHAR_TYPE_NORMAL:
 	default:
 		prefix = gok_wordcomplete_unichar_push (complete, letter);
@@ -432,6 +634,24 @@ gok_wordcomplete_real_get_word_part (GokWordComplete *complete)
 		return NULL;
 }
 
+/**
+ **/
+static gchar *
+gok_wordcomplete_real_get_aux_dict (GokWordComplete *complete)
+{
+	return complete->priv->aux_dictionaries;
+}
+
+/**
+ **/
+static gboolean
+gok_wordcomplete_real_set_aux_dict (GokWordComplete *complete, const gchar *dict_path)
+{
+	if (complete && complete->priv)
+		complete->priv->aux_dictionaries = dict_path;
+	return TRUE;
+}
+
 /**
 * gok_wordcomplete_real_predict
 * 
