@@ -155,7 +155,7 @@ WordCompleteOpen(GokWordComplete *complete, const gchar *directory)
 
 	/* open the dictionary file */
 	dictionary_full_path = g_build_filename (directory, "dictionary.txt", NULL);
-	pFile = fopen (dictionary_full_path, "rt");
+	pFile = fopen (dictionary_full_path, "r");
 	if (pFile == NULL)
 	{
 		/* can't open dictionary file! */
@@ -590,6 +590,8 @@ WordCompletePredictString (GokWordComplete *complete, const gchar* pWord, gint n
 		bMadePrediction = TRUE;
 	}
 
+	word_predict_list[i] = NULL;
+
 	/* were there any predictions? */
 	if (bMadePrediction == FALSE)
 	{
